@@ -5,7 +5,7 @@ import inspect
 from typing import Any, Callable, NamedTuple, List, Union
 
 from cloc.utils import defaultattr
-from cloc.types import BaseParamType
+from cloc.types import BaseType
 
 '''
 core
@@ -22,10 +22,12 @@ class BaseArg(object):
 
 class Opt(BaseArg):
     short_name: str
+    default: Any
 
-    def __init__(self, name: str, short_name: str, type: Any = None, help: str = None):
+    def __init__(self, name: str, short_name: str, type: Any = None, default: Any= None, help: str = None):
         super().__init__(name, type, help)
         defaultattr(self, 'short_name', short_name)
+        defaultattr(self, 'default', default)
 
 class Flg(BaseArg):
     short_name: str
@@ -114,30 +116,32 @@ class Cmd(BaseCmd):
             for p in self.params.order:
                 if isinstance(p, Arg):
                     usagestr += f'{p.name} '
+                    paramstr = f'{p.name} '
                     paramstr += f'{type(p).__name__!r} '
                     if p.type:
-                        paramstr += f'\t{p.type.__name__!r} '
-                    paramstr += f'\t{p.name} '
+                        paramstr += f'{p.type.__name__!r} '
                     if p.help:
-                        paramstr += f'\t{p.help} '
+                        paramstr += f'{p.help} '
                     paramstr += '\n'
                 if isinstance(p, Opt):
                     usagestr += f'{p.name}|{p.short_name} [value] '
+                    paramstr += f'{p.name} {p.short_name} '
                     paramstr += f'{type(p).__name__!r} '
                     if p.type:
-                        paramstr += f'\t{p.type.__name__!r} '
-                    paramstr += f'\t{p.name} {p.short_name} '
+                        paramstr += f'{p.type.__name__!r} '
+                    if p.default:
+                        paramstr += f'[default: {str(p.default)}] '
                     if p.help:
-                        paramstr += f'\t{p.help} '
+                        paramstr += f'{p.help} '
                     paramstr += '\n'
                 if isinstance(p,  Flg):
                     usagestr += f'{p.name}|{p.short_name} '
+                    paramstr += f'{p.name} {p.short_name} '
                     paramstr += f'{type(p).__name__!r} '
                     if p.type:
-                        paramstr += f'\t{p.type.__name__!r} (flag) '
-                    paramstr += f'\t{p.name} {p.short_name} '
+                        paramstr += f'{p.type.__name__!r} (flag) '
                     if p.help:
-                        paramstr += f'\t{p.help} '
+                        paramstr += f'{p.help} '
                     paramstr += '\n'
 
         usagestr += '\n'
@@ -157,6 +161,13 @@ class Cmd(BaseCmd):
                         rgx_pattern += f'-{p.short_name.replace("-", "")})'
                     self.regex_patterns.insert(0, rgx_pattern)
 
+    def _convert_type(self, value: Any, index: int):
+        if 'builtins' == self.params.order[index].type.__class__.__module__:
+            value = self.params.order[index].type(value)
+        elif isinstance(self.params.order[index].type, BaseType):
+            value = self.params.order[index].type.convert(value)
+        return value
+
     def get_params_values(self, cmdl: list):
         if '--help' in cmdl:
             self._print_help()
@@ -170,27 +181,25 @@ class Cmd(BaseCmd):
                         msg += f'instead of type {"arg"!r}. Order of cmd parameters might be incorrect.'
                         raise TypeError(msg)
                     if cmdl[index]:
-                        if 'builtins' == self.params.order[index].type.__class__.__module__:
-                            self.values.append(self.params.order[index].type(cmdl[index]))
-                        elif isinstance(self.params.order[index].type, BaseParamType):
-                            self.values.append(self.params.order[index].type.convert(cmdl[index]))
-                        else:
-                            self.values.append(cmdl[index])
+                        self.values.append(self._convert_type(cmdl[index], index))
                 if isinstance(self.params.order[index], Opt):
                     matches = re.findall(self.regex_patterns[index], ' '.join(cmdl))
                     if matches:
                         for m in matches:
                             if m[1]:
-                                if 'builtins' == self.params.order[index].type.__class__.__module__:
-                                    self.values.append(self.params.order[index].type(m[1]))
-                                elif isinstance(self.params.order[index].type, BaseParamType):
-                                    self.values.append(self.params.order[index].type.convert(m[1]))
-                                else:
-                                    self.values.append(m[1])
+                                self.values.append(self._convert_type(m[1], index))
+                    else:
+                        if not self.params.order[index].default:
+                            msg = f'{self.params.order[index].name} was not received or given a default value'
+                            raise TypeError(msg)
+                        else:
+                            self.values.append(self._convert_type(self.params.order[index].default, index))
                 if isinstance(self.params.order[index], Flg):
                     matches = re.findall(self.regex_patterns[index], ' '.join(cmdl))
                     if matches:
                         self.values.append(True)
+                    else:
+                        self.values.append(False)
 
 class Grp(BaseCmd):
     commands: List[Cmd]
@@ -221,6 +230,7 @@ class Grp(BaseCmd):
                 method = getattr(command, method_name)
                 if isinstance(method, (Grp, Cmd)):
                     cmd = method.new_dataclass_cmd(method.name, method.fn, method.params, method.hidden, command)
+                    cmd.__doc__ = method.__doc__
                     if hidden:
                         cmd.hidden = hidden
                     self.commands.append(cmd)
@@ -245,7 +255,7 @@ class Grp(BaseCmd):
         if hasattr(self, 'commands'):
             for c in self.commands:
                 if not c.hidden:
-                    cmdstr += f'{c.name}\n'
+                    cmdstr += f'{c.name}\t\t{c.__doc__}\n'
         self.help = usagestr + docstr + cmdstr
 
     def get_params_values(self, cmdl: list):
